@@ -107,44 +107,72 @@ pub fn forward_derivative(
                 );
                 return Some(new_stmt);
             } else if let syn::Expr::Path(expr_path) = &*init.1 {
+                // Given `let x = y;`
+
+                // This is `x`
                 let out_ident = local
                     .pat
                     .ident()
                     .expect("forward_add: not ident")
                     .ident
                     .to_string();
+                // This `y`
                 let in_ident = expr_path.path.segments[0].ident.to_string();
-                let out_type = type_map.get(&in_ident).expect("forward_derivative: return unfound type");
-                let return_type = rust_ad_core::Type::try_from(out_type.as_str()).expect("unsupported return type");
-                let return_derivative = match &*init.1 {
-                    // Result 1
-                    syn::Expr::Lit(_) => return_type.zero(),
-                    syn::Expr::Path(path_expr) => {
-                        // x typically is the left or right of binary expression, regardless we are doing d/dx(expr) so at this we got
-                        let x = path_expr.path.segments[0].ident.to_string();
-            
-                        // Result 3
-                        if x == in_ident {
-                            der!(in_ident)
-                        }
-                        // Result 4
-                        else if function_inputs.contains(&x) {
-                            return_type.zero()
-                        }
-                        // Result 2
-                        else {
-                            wrt!(x, in_ident)
-                        }
-                    }
-                    _ => panic!("cumulative_derivative_wrt: unsupported expr"),
-                };
-                let new_stmt_str = format!(
-                    "let {} = {};",
-                    wrt!(out_ident, in_ident),
-                    return_derivative
-                );
+                // This is type of `y`
+                let out_type = type_map
+                    .get(&in_ident)
+                    .expect("forward_derivative: return unfound type");
+                let return_type = rust_ad_core::Type::try_from(out_type.as_str())
+                    .expect("forward_derivative: unsupported return type");
+
+                let idents = function_inputs
+                    .iter()
+                    .map(|input| wrt!(out_ident, input))
+                    .intersperse(String::from(","))
+                    .collect::<String>();
+                let deriatives = function_inputs
+                    .iter()
+                    .map(|input| {
+                        cumulative_derivative_wrt_rt(&*init.1, input, function_inputs, &return_type)
+                    })
+                    .intersperse(String::from(","))
+                    .collect::<String>();
+                let stmt_str = format!("let ({}) = ({});", idents, deriatives);
                 let new_stmt: syn::Stmt =
-                    syn::parse_str(&new_stmt_str).expect("forward_derivative: parse fail");
+                    syn::parse_str(&stmt_str).expect("forward_derivative: parse fail");
+
+                return Some(new_stmt);
+            } else if let syn::Expr::Lit(expr_lit) = &*init.1 {
+                // Given `let x = y;`
+
+                // This is `x`
+                let out_ident = local
+                    .pat
+                    .ident()
+                    .expect("forward_add: not ident")
+                    .ident
+                    .to_string();
+                // This is type of `y`
+                let out_type = literal_type(expr_lit).expect("forward_derivative: bad lit type");
+                let return_type = rust_ad_core::Type::try_from(out_type.as_str())
+                    .expect("forward_derivative: unsupported return type");
+
+                let idents = function_inputs
+                    .iter()
+                    .map(|input| wrt!(out_ident, input))
+                    .intersperse(String::from(","))
+                    .collect::<String>();
+                let deriatives = function_inputs
+                    .iter()
+                    .map(|input| {
+                        cumulative_derivative_wrt_rt(&*init.1, input, function_inputs, &return_type)
+                    })
+                    .intersperse(String::from(","))
+                    .collect::<String>();
+                let stmt_str = format!("let ({}) = ({});", idents, deriatives);
+                let new_stmt: syn::Stmt =
+                    syn::parse_str(&stmt_str).expect("forward_derivative: parse fail");
+
                 return Some(new_stmt);
             }
         }
